@@ -21,8 +21,8 @@ Information about setting up your Google OAUTH2 credentials:
 For libcloud, there are two basic methods for authenticating to Google using
 OAUTH2: Service Accounts and Client IDs for Installed Applications.
 
-Both are initially set up from the Cloud Console_
-_Console: https://cloud.google.com/console
+Both are initially set up from the Cloud Console Console -
+https://cloud.google.com/console
 
 Setting up Service Account authentication (note that you need the PyCrypto
 package installed to use this):
@@ -38,9 +38,7 @@ package installed to use this):
 - Optionally, you may choose to Generate a PKCS12 key from the Console.
   It needs to be converted to the PEM format.  Please note, the PKCS12 format
   is deprecated and may be removed in a future release.
-  - Convert the key using OpenSSL (the default password is 'notasecret'):
-    ``openssl pkcs12 -in YOURPRIVKEY.p12 -nodes -nocerts
-    -passin pass:notasecret | openssl rsa -out PRIV.pem``
+  - Convert the key using OpenSSL (the default password is 'notasecret').
   - Move the .pem file to a safe location.
 - To Authenticate, you will need to pass the Service Account's "Email
   address" in as the user_id and the path to the .pem file as the key.
@@ -66,6 +64,7 @@ Setting up Installed Application authentication:
 
 Please remember to secure your keys and access tokens.
 """
+
 from __future__ import with_statement
 
 try:
@@ -73,11 +72,11 @@ try:
 except ImportError:
     import json
 
+import logging
 import base64
 import errno
 import time
 import datetime
-import logging
 import os
 import socket
 import sys
@@ -103,6 +102,8 @@ except ImportError:
 
 UTC_TIMESTAMP_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
 
+LOG = logging.getLogger(__name__)
+
 
 def _utcnow():
     """
@@ -112,10 +113,16 @@ def _utcnow():
 
 
 def _utc_timestamp(datetime_obj):
+    """
+    Return string of datetime_obj in the UTC Timestamp Format
+    """
     return datetime_obj.strftime(UTC_TIMESTAMP_FORMAT)
 
 
 def _from_utc_timestamp(timestamp):
+    """
+    Return datetime obj where date and time are pulled from timestamp string.
+    """
     return datetime.datetime.strptime(timestamp, UTC_TIMESTAMP_FORMAT)
 
 
@@ -162,7 +169,7 @@ class ResourceNotFoundError(GoogleBaseError):
                 "error may be an authentication issue. " \
                 "Please  ensure your auth credentials match " \
                 "your project. "
-        super(GoogleBaseError, self).__init__(value, http_code, driver)
+        super(ResourceNotFoundError, self).__init__(value, http_code, driver)
 
 
 class QuotaExceededError(GoogleBaseError):
@@ -213,7 +220,9 @@ class GoogleResponse(JsonResponse):
             code = err.get('code')
             message = err.get('message')
         else:
-            code = err.get('reason', None)
+            code = None
+            if 'reason' in err:
+                code = err.get('reason')
             message = body.get('error_description', err)
 
         return (code, message)
@@ -337,6 +346,9 @@ class GoogleBaseAuthConnection(ConnectionUserAndKey):
         super(GoogleBaseAuthConnection, self).__init__(user_id, key, **kwargs)
 
     def add_default_headers(self, headers):
+        """
+        Add defaults for 'Content-Type' and 'Host' headers.
+        """
         headers['Content-Type'] = "application/x-www-form-urlencoded"
         headers['Host'] = self.host
         return headers
@@ -379,6 +391,7 @@ class GoogleBaseAuthConnection(ConnectionUserAndKey):
         :return:  A dictionary containing updated token information.
         :rtype:   ``dict``
         """
+        # pylint: disable=no-member
         return self.get_new_token()
 
 
@@ -487,10 +500,6 @@ class GoogleServiceAcctAuthConnection(GoogleBaseAuthConnection):
                 key = key['private_key']
             except ValueError:
                 key = contents
-                logger = logging.getLogger(__name__)
-                logger.warn('%s not in JSON format.  This format is '
-                            'deprecated.  Please download a JSON key '
-                            'from the Cloud Console.' % keypath)
 
         super(GoogleServiceAcctAuthConnection, self).__init__(
             user_id, key, *args, **kwargs)
@@ -532,7 +541,7 @@ class GoogleServiceAcctAuthConnection(GoogleBaseAuthConnection):
 
 class GoogleGCEServiceAcctAuthConnection(GoogleBaseAuthConnection):
     """Authentication class for self-authentication when used with a GCE
-    istance that supports serviceAccounts.
+    instance that supports serviceAccounts.
     """
     def get_new_token(self):
         """
@@ -576,10 +585,10 @@ class GoogleAuthType(object):
     def guess_type(cls, user_id):
         if cls._is_sa(user_id):
             return cls.SA
-        elif cls._is_gce():
-            return cls.GCE
         elif cls._is_gcs_s3(user_id):
             return cls.GCS_S3
+        elif cls._is_gce():
+            return cls.GCE
         else:
             return cls.IA
 
@@ -610,6 +619,110 @@ class GoogleAuthType(object):
         return user_id.endswith('.gserviceaccount.com')
 
 
+class GoogleOAuth2Credential(object):
+    default_credential_file = '~/.google_libcloud_auth'
+
+    def __init__(self, user_id, key, auth_type=None, credential_file=None,
+                 scopes=None, **kwargs):
+        self.auth_type = auth_type or GoogleAuthType.guess_type(user_id)
+        if self.auth_type not in GoogleAuthType.ALL_TYPES:
+            raise GoogleAuthError('Invalid auth type: %s' % self.auth_type)
+        if not GoogleAuthType.is_oauth2(self.auth_type):
+            raise GoogleAuthError(('Auth type %s cannot be used with OAuth2' %
+                                   self.auth_type))
+        self.user_id = user_id
+        self.key = key
+
+        default_credential_file = '.'.join([self.default_credential_file,
+                                            user_id])
+        self.credential_file = credential_file or default_credential_file
+        # Default scopes to read/write for compute, storage, and dns.
+        self.scopes = scopes or [
+            'https://www.googleapis.com/auth/compute',
+            'https://www.googleapis.com/auth/devstorage.full_control',
+            'https://www.googleapis.com/auth/ndev.clouddns.readwrite',
+        ]
+
+        self.token = self._get_token_from_file()
+
+        if self.auth_type == GoogleAuthType.GCE:
+            self.oauth2_conn = GoogleGCEServiceAcctAuthConnection(
+                self.user_id, self.scopes, **kwargs)
+        elif self.auth_type == GoogleAuthType.SA:
+            self.oauth2_conn = GoogleServiceAcctAuthConnection(
+                self.user_id, self.key, self.scopes, **kwargs)
+        elif self.auth_type == GoogleAuthType.IA:
+            self.oauth2_conn = GoogleInstalledAppAuthConnection(
+                self.user_id, self.key, self.scopes, **kwargs)
+        else:
+            raise GoogleAuthError('Invalid auth_type: %s' %
+                                  str(self.auth_type))
+
+        if self.token is None:
+            self.token = self.oauth2_conn.get_new_token()
+            self._write_token_to_file()
+
+    @property
+    def access_token(self):
+        if self.token_expire_utc_datetime < _utcnow():
+            self._refresh_token()
+        return self.token['access_token']
+
+    @property
+    def token_expire_utc_datetime(self):
+        return _from_utc_timestamp(self.token['expire_time'])
+
+    def _refresh_token(self):
+        self.token = self.oauth2_conn.refresh_token(self.token)
+        self._write_token_to_file()
+
+    def _get_token_from_file(self):
+        """
+        Read credential file and return token information.
+        Mocked in libcloud.test.common.google.GoogleTestCase.
+
+        :return:  Token information dictionary, or None
+        :rtype:   ``dict`` or ``None``
+        """
+        token = None
+        filename = os.path.realpath(os.path.expanduser(self.credential_file))
+
+        try:
+            with open(filename, 'r') as f:
+                data = f.read()
+            token = json.loads(data)
+        except (IOError, ValueError):
+            # Note: File related errors (IOError) and errors related to json
+            # parsing of the data (ValueError) are not fatal.
+            e = sys.exc_info()[1]
+            LOG.info('Failed to read cached auth token from file "%s": %s',
+                     filename, str(e))
+
+        return token
+
+    def _write_token_to_file(self):
+        """
+        Write token to credential file.
+        Mocked in libcloud.test.common.google.GoogleTestCase.
+        """
+        filename = os.path.expanduser(self.credential_file)
+        filename = os.path.realpath(filename)
+
+        try:
+            data = json.dumps(self.token)
+            write_flags = os.O_CREAT | os.O_WRONLY | os.O_TRUNC
+            with os.fdopen(os.open(filename, write_flags,
+                                   int('600', 8)), 'w') as f:
+                f.write(data)
+        except:
+            # Note: Failure to write (cache) token in a file is not fatal. It
+            # simply means degraded performance since we will need to acquire a
+            # new token each time script runs.
+            e = sys.exc_info()[1]
+            LOG.info('Failed to write auth token to file "%s": %s',
+                     filename, str(e))
+
+
 class GoogleBaseConnection(ConnectionUserAndKey, PollingConnection):
     """Base connection class for interacting with Google APIs."""
     driver = GoogleBaseDriver
@@ -617,7 +730,6 @@ class GoogleBaseConnection(ConnectionUserAndKey, PollingConnection):
     host = 'www.googleapis.com'
     poll_interval = 2.0
     timeout = 180
-    credential_file = '~/.google_libcloud_auth'
 
     def __init__(self, user_id, key=None, auth_type=None,
                  credential_file=None, scopes=None, **kwargs):
@@ -649,35 +761,16 @@ class GoogleBaseConnection(ConnectionUserAndKey, PollingConnection):
                           read/write access to Compute, Storage, and DNS.
         :type     scopes: ``list``
         """
-        self.user_id = user_id
-        self.key = key
-        if auth_type and auth_type not in GoogleAuthType.ALL_TYPES:
-            raise GoogleAuthError('Invalid auth type: %s' % auth_type)
-        self.auth_type = auth_type or GoogleAuthType.guess_type(user_id)
-
-        # OAuth2 stuff and placeholders
-        self.scopes = scopes
-        self.oauth2_conn = None
-        self.oauth2_token = None
-        if credential_file:
-            self.credential_file = credential_file
-        elif self.auth_type == GoogleAuthType.SA:
-            self.credential_file += '.' + user_id
-
-        if GoogleAuthType.is_oauth2(self.auth_type):
-            self._init_oauth2(**kwargs)
-
         super(GoogleBaseConnection, self).__init__(user_id, key, **kwargs)
 
+        self.oauth2_credential = GoogleOAuth2Credential(
+            user_id, key, auth_type, credential_file, scopes, **kwargs)
+
         python_ver = '%s.%s.%s' % (sys.version_info[0], sys.version_info[1],
                                    sys.version_info[2])
         ver_platform = 'Python %s/%s' % (python_ver, sys.platform)
         self.user_agent_append(ver_platform)
 
-    @property
-    def token_expire_utc_datetime(self):
-        return _from_utc_timestamp(self.oauth2_token['expire_time'])
-
     def add_default_headers(self, headers):
         """
         @inherits: :class:`Connection.add_default_headers`
@@ -693,11 +786,8 @@ class GoogleBaseConnection(ConnectionUserAndKey, PollingConnection):
 
         @inherits: :class:`Connection.pre_connect_hook`
         """
-        if self.token_expire_utc_datetime < _utcnow():
-            self._refresh_oauth2_token()
-        headers['Authorization'] = 'Bearer %s' % (
-            self.oauth2_token['access_token'])
-
+        headers['Authorization'] = ('Bearer ' +
+                                    self.oauth2_credential.access_token)
         return params, headers
 
     def encode_data(self, data):
@@ -767,64 +857,3 @@ class GoogleBaseConnection(ConnectionUserAndKey, PollingConnection):
         else:
             request = self.request_path + action
         return request
-
-    def _refresh_oauth2_token(self):
-        self.oauth2_token = self.oauth2_conn.refresh_token(self.oauth2_token)
-        self._write_token_to_file()
-
-    def _init_oauth2(self, **kwargs):
-        # Default scopes to read/write for compute, storage, and dns.  Can
-        # override this when calling get_driver() or setting in secrets.py
-        if not self.scopes:
-            self.scopes = [
-                'https://www.googleapis.com/auth/compute',
-                'https://www.googleapis.com/auth/devstorage.full_control',
-                'https://www.googleapis.com/auth/ndev.clouddns.readwrite',
-            ]
-        self.oauth2_token = self._get_token_from_file()
-
-        if self.auth_type == GoogleAuthType.GCE:
-            self.oauth2_conn = GoogleGCEServiceAcctAuthConnection(
-                self.user_id, self.scopes, **kwargs)
-        elif self.auth_type == GoogleAuthType.SA:
-            self.oauth2_conn = GoogleServiceAcctAuthConnection(
-                self.user_id, self.key, self.scopes, **kwargs)
-        elif self.auth_type == GoogleAuthType.IA:
-            self.oauth2_conn = GoogleInstalledAppAuthConnection(
-                self.user_id, self.key, self.scopes, **kwargs)
-        else:
-            raise GoogleAuthError('Invalid auth_type: %s' %
-                                  str(self.auth_type))
-
-        if self.oauth2_token is None:
-            self.oauth2_token = self.oauth2_conn.get_new_token()
-            self._write_token_to_file()
-
-    def _get_token_from_file(self):
-        """
-        Read credential file and return token information.
-        Mocked in libcloud.test.common.google.GoogleTestCase.
-
-        :return:  Token information dictionary, or None
-        :rtype:   ``dict`` or ``None``
-        """
-        token = None
-        filename = os.path.realpath(os.path.expanduser(self.credential_file))
-
-        try:
-            with open(filename, 'r') as f:
-                data = f.read()
-            token = json.loads(data)
-        except IOError:
-            pass
-        return token
-
-    def _write_token_to_file(self):
-        """
-        Write token to credential file.
-        Mocked in libcloud.test.common.google.GoogleTestCase.
-        """
-        filename = os.path.realpath(os.path.expanduser(self.credential_file))
-        data = json.dumps(self.oauth2_token)
-        with open(filename, 'w') as f:
-            f.write(data)
