@@ -820,6 +820,35 @@ class ECSDriver(NodeDriver):
         resp = self.connection.request(self.path, params)
         return resp.success()
 
+    def ex_modify_security_group_by_id(
+            self,
+            group_id=None,
+            name=None,
+            description=None):
+        """
+        Modify a new security group.
+        :keyword group_id: id of the security group
+        :type group_id: ``str``
+        :keyword name: new name of the security group
+        :type name: ``unicode``
+        :keyword description: new description of the security group
+        :type description: ``unicode``
+        """
+
+        params = {'Action': 'ModifySecurityGroupAttribute',
+                  'RegionId': self.region}
+        if not group_id:
+            raise AttributeError('group_id is required')
+        params["SecurityGroupId"] = group_id
+
+        if name:
+            params["SecurityGroupName"] = name
+        if description:
+            params["Description"] = description
+
+        resp = self.connection.request(self.path, params)
+        return resp.success()
+
     def ex_list_security_groups(self, ex_filters=None):
         """
         List security groups in the current region.
