@@ -19,12 +19,7 @@ import base64
 import os
 import binascii
 
-try:
-    from lxml import etree as ET
-except ImportError:
-    from xml.etree import ElementTree as ET
-
-from libcloud.utils.py3 import PY3
+from libcloud.utils.py3 import ET
 from libcloud.utils.py3 import httplib
 from libcloud.utils.py3 import urlquote
 from libcloud.utils.py3 import tostring
@@ -43,9 +38,6 @@ from libcloud.storage.types import ContainerDoesNotExistError
 from libcloud.storage.types import ObjectDoesNotExistError
 from libcloud.storage.types import ObjectHashMismatchError
 
-if PY3:
-    from io import FileIO as file
-
 # Desired number of items in each response inside a paginated request
 RESPONSES_PER_REQUEST = 100
 
@@ -389,7 +381,7 @@ class AzureBlobsStorageDriver(StorageDriver):
             if not params['marker']:
                 break
 
-    def iterate_container_objects(self, container):
+    def iterate_container_objects(self, container, ex_prefix=None):
         """
         @inherits: :class:`StorageDriver.iterate_container_objects`
         """
@@ -398,6 +390,9 @@ class AzureBlobsStorageDriver(StorageDriver):
                   'maxresults': RESPONSES_PER_REQUEST,
                   'include': 'metadata'}
 
+        if ex_prefix:
+            params['prefix'] = ex_prefix
+
         container_path = self._get_container_path(container)
 
         while True:
@@ -424,6 +419,22 @@ class AzureBlobsStorageDriver(StorageDriver):
             if not params['marker']:
                 break
 
+    def list_container_objects(self, container, ex_prefix=None):
+        """
+        Return a list of objects for the given container.
+
+        :param container: Container instance.
+        :type container: :class:`Container`
+
+        :param ex_prefix: Only return objects starting with ex_prefix
+        :type ex_prefix: ``str``
+
+        :return: A list of Object instances.
+        :rtype: ``list`` of :class:`Object`
+        """
+        return list(self.iterate_container_objects(container,
+                                                   ex_prefix=ex_prefix))
+
     def get_container(self, container_name):
         """
         @inherits: :class:`StorageDriver.get_container`
@@ -568,11 +579,14 @@ class AzureBlobsStorageDriver(StorageDriver):
         @inherits: :class:`StorageDriver.download_object_as_stream`
         """
         obj_path = self._get_object_path(obj.container, obj.name)
-        response = self.connection.request(obj_path, raw=True, data=None)
+        response = self.connection.request(obj_path, method='GET',
+                                           stream=True, raw=True)
+        iterator = response.iter_content(AZURE_CHUNK_SIZE)
 
-        return self._get_object(obj=obj, callback=read_in_chunks,
+        return self._get_object(obj=obj,
+                                callback=read_in_chunks,
                                 response=response,
-                                callback_kwargs={'iterator': response.response,
+                                callback_kwargs={'iterator': iterator,
                                                  'chunk_size': chunk_size},
                                 success_status_code=httplib.OK)
 
@@ -642,7 +656,7 @@ class AzureBlobsStorageDriver(StorageDriver):
             chunk_hash = base64.b64encode(b(chunk_hash.digest()))
 
             headers['Content-MD5'] = chunk_hash.decode('utf-8')
-            headers['Content-Length'] = content_length
+            headers['Content-Length'] = str(content_length)
 
             if blob_type == 'BlockBlob':
                 # Block id can be any unique string that is base64 encoded
@@ -764,38 +778,19 @@ class AzureBlobsStorageDriver(StorageDriver):
 
         self._check_values(ex_blob_type, file_size)
 
-        with file(file_path, 'rb') as file_handle:
-            iterator = iter(file_handle)
-
-            # If size is greater than 64MB or type is Page, upload in chunks
-            if ex_blob_type == 'PageBlob' or file_size > AZURE_BLOCK_MAX_SIZE:
-                # For chunked upload of block blobs, the initial size must
-                # be 0.
-                if ex_blob_type == 'BlockBlob':
-                    object_size = None
-
-                object_path = self._get_object_path(container, object_name)
-
-                upload_func = self._upload_in_chunks
-                upload_func_kwargs = {'iterator': iterator,
-                                      'object_path': object_path,
-                                      'blob_type': ex_blob_type,
-                                      'lease': None}
-            else:
-                upload_func = self._stream_data
-                upload_func_kwargs = {'iterator': iterator,
-                                      'chunked': False,
-                                      'calculate_hash': verify_hash}
-
-            return self._put_object(container=container,
-                                    object_name=object_name,
-                                    object_size=object_size,
-                                    upload_func=upload_func,
-                                    upload_func_kwargs=upload_func_kwargs,
-                                    file_path=file_path, extra=extra,
-                                    verify_hash=verify_hash,
-                                    blob_type=ex_blob_type,
-                                    use_lease=ex_use_lease)
+        # If size is greater than 64MB or type is Page, upload in chunks
+        if ex_blob_type == 'PageBlob' or file_size > AZURE_BLOCK_MAX_SIZE:
+            # For chunked upload of block blobs, the initial size must
+            # be 0.
+            if ex_blob_type == 'BlockBlob':
+                object_size = None
+        return self._put_object(container=container,
+                                object_name=object_name,
+                                object_size=object_size,
+                                file_path=file_path, extra=extra,
+                                verify_hash=verify_hash,
+                                blob_type=ex_blob_type,
+                                use_lease=ex_use_lease)
 
     def upload_object_via_stream(self, iterator, container, object_name,
                                  verify_hash=False, extra=None,
@@ -818,24 +813,31 @@ class AzureBlobsStorageDriver(StorageDriver):
         if ex_blob_type is None:
             ex_blob_type = self.ex_blob_type
 
+        """
+        Azure requires that for page blobs that a maximum size that the page
+        can grow to.  For block blobs, it is required that the Content-Length
+        header be set.  The size of the block blob will be the total size of
+        the stream minus the current position, so in this case
+        ex_page_blob_size should be 0 (and will be checked in
+        self._check_values).
+        Source:
+        https://docs.microsoft.com/en-us/rest/api/storageservices/put-blob
+        """
         self._check_values(ex_blob_type, ex_page_blob_size)
-
-        object_path = self._get_object_path(container, object_name)
-
-        upload_func = self._upload_in_chunks
-        upload_func_kwargs = {'iterator': iterator,
-                              'object_path': object_path,
-                              'blob_type': ex_blob_type,
-                              'lease': None}
+        if ex_blob_type == "BlockBlob":
+            iterator.seek(0, os.SEEK_END)
+            blob_size = iterator.tell()
+            iterator.seek(0)
+        else:
+            blob_size = ex_page_blob_size
 
         return self._put_object(container=container,
                                 object_name=object_name,
-                                object_size=ex_page_blob_size,
-                                upload_func=upload_func,
-                                upload_func_kwargs=upload_func_kwargs,
+                                object_size=blob_size,
                                 extra=extra, verify_hash=verify_hash,
                                 blob_type=ex_blob_type,
-                                use_lease=ex_use_lease)
+                                use_lease=ex_use_lease,
+                                stream=iterator)
 
     def delete_object(self, obj):
         """
@@ -897,17 +899,18 @@ class AzureBlobsStorageDriver(StorageDriver):
         self._update_metadata(headers, meta_data)
 
         if object_size is not None:
-            headers['Content-Length'] = object_size
+            headers['Content-Length'] = str(object_size)
 
         if blob_type == 'PageBlob':
-            headers['Content-Length'] = 0
-            headers['x-ms-blob-content-length'] = object_size
+            headers['Content-Length'] = str('0')
+            headers['x-ms-blob-content-length'] = str(object_size)
 
         return headers
 
-    def _put_object(self, container, object_name, object_size, upload_func,
-                    upload_func_kwargs, file_path=None, extra=None,
-                    verify_hash=True, blob_type=None, use_lease=False):
+    def _put_object(self, container, object_name, object_size,
+                    file_path=None, extra=None,
+                    verify_hash=True, blob_type=None, use_lease=False,
+                    stream=None):
         """
         Control function that does the real job of uploading data to a blob
         """
@@ -922,23 +925,17 @@ class AzureBlobsStorageDriver(StorageDriver):
 
         # Get a lease if required and do the operations
         with AzureBlobLease(self, object_path, use_lease) as lease:
-            if 'lease' in upload_func_kwargs:
-                upload_func_kwargs['lease'] = lease
-
             lease.update_headers(headers)
 
-            iterator = iter('')
             result_dict = self._upload_object(object_name, content_type,
-                                              upload_func, upload_func_kwargs,
                                               object_path, headers=headers,
                                               file_path=file_path,
-                                              iterator=iterator)
+                                              stream=stream)
 
             response = result_dict['response']
             bytes_transferred = result_dict['bytes_transferred']
             data_hash = result_dict['data_hash']
             headers = response.headers
-            response = response.response
 
         if response.status != httplib.CREATED:
             raise LibcloudError(
