@@ -329,10 +329,12 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
 
     def test_authorize_security_group_ingress(self):
         ranges = ['1.1.1.1/32', '2.2.2.2/32']
-        resp = self.driver.ex_authorize_security_group_ingress('sg-42916629', 22, 22, cidr_ips=ranges)
+        description = "automated authorised IP ingress test"
+        resp = self.driver.ex_authorize_security_group_ingress('sg-42916629', 22, 22, cidr_ips=ranges, description=description)
         self.assertTrue(resp)
         groups = [{'group_id': 'sg-949265ff'}]
-        resp = self.driver.ex_authorize_security_group_ingress('sg-42916629', 22, 23, group_pairs=groups)
+        description = "automated authorised group ingress test"
+        resp = self.driver.ex_authorize_security_group_ingress('sg-42916629', 22, 23, group_pairs=groups, description=description)
         self.assertTrue(resp)
 
     def test_authorize_security_group_egress(self):
@@ -433,39 +435,7 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
             self.driver.api_name = api_name
             self.driver.region_name = region_name
             sizes = self.driver.list_sizes()
-
-            ids = [s.id for s in sizes]
-
-            if region_name not in ['ap-south-1']:
-                self.assertTrue('t1.micro' in ids)
-                self.assertTrue('m1.small' in ids)
-                self.assertTrue('m1.large' in ids)
-                self.assertTrue('m1.xlarge' in ids)
-                self.assertTrue('c1.medium' in ids)
-                self.assertTrue('c1.xlarge' in ids)
-                self.assertTrue('m2.xlarge' in ids)
-                self.assertTrue('m2.2xlarge' in ids)
-                self.assertTrue('m2.4xlarge' in ids)
-
-            if region_name == 'us-east-1':
-                self.assertEqual(len(sizes), 84)
-                self.assertTrue('cg1.4xlarge' in ids)
-                self.assertTrue('cc2.8xlarge' in ids)
-                self.assertTrue('cr1.8xlarge' in ids)
-                self.assertTrue('x1.32xlarge' in ids)
-            elif region_name == 'us-west-1':
-                self.assertEqual(len(sizes), 67)
-            if region_name == 'us-west-2':
-                self.assertEqual(len(sizes), 79)
-            elif region_name == 'ap-southeast-1':
-                self.assertEqual(len(sizes), 59)
-            elif region_name == 'ap-southeast-2':
-                self.assertEqual(len(sizes), 63)
-            elif region_name == 'eu-west-1':
-                self.assertEqual(len(sizes), 82)
-            elif region_name == 'ap-south-1':
-                self.assertEqual(len(sizes), 41)
-
+            self.assertNotEqual(len(sizes), 0)
         self.driver.region_name = region_old
 
     def test_ex_create_node_with_ex_iam_profile(self):
@@ -963,6 +933,14 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         # 2013-08-15T16:22:30.000Z
         self.assertEqual(datetime(2013, 8, 15, 16, 22, 30, tzinfo=UTC), snap.created)
 
+    def test_create_volume_snapshot_with_tags(self):
+        vol = StorageVolume(id='vol-4282672b', name='test',
+                            state=StorageVolumeState.AVAILABLE,
+                            size=10, driver=self.driver)
+        snap = self.driver.create_volume_snapshot(
+            vol, 'Test snapshot', ex_metadata={'my_tag': 'test'})
+        self.assertEqual('test', snap.extra['tags']['my_tag'])
+
     def test_list_snapshots(self):
         snaps = self.driver.list_snapshots()
 
@@ -1824,6 +1802,14 @@ class NimbusTests(EC2Tests):
         self.driver.ex_create_tags(resource=node, tags={'foo': 'bar'})
         self.assertExecutedMethodCount(0)
 
+    def test_create_volume_snapshot_with_tags(self):
+        vol = StorageVolume(id='vol-4282672b', name='test',
+                            state=StorageVolumeState.AVAILABLE,
+                            size=10, driver=self.driver)
+        snap = self.driver.create_volume_snapshot(
+            vol, 'Test snapshot', ex_metadata={'my_tag': 'test'})
+        self.assertDictEqual({}, snap.extra['tags'])
+
 
 class EucTests(LibcloudTestCase, TestCaseMixin):
 
